/**
 * Simple class representing a plaintext/ciphertext pair.
 */
public class TextPair {
    private static final int UNKNOWN = -1;

    private int ina, inb, outx, outy;

    public TextPair() {
        set(UNKNOWN, UNKNOWN, UNKNOWN, UNKNOWN);
    }

    public TextPair(int ina, int inb) {
        set(ina, inb, UNKNOWN, UNKNOWN);
    }

    public TextPair(int ina, int inb, int outx, int outy) {
        set(ina, inb, outx, outy);
    }

    public void set(int ina, int inb) {
        set(ina, inb, UNKNOWN, UNKNOWN);
    }

    public void set(int ina, int inb, int outx, int outy) {
        this.ina = ina;
        this.inb = inb;
        this.outx = outx;
        this.outy = outy;
    }

    public int getInA() { return ina; }
    public int getInB() { return inb; }
    public int getOutX() { return outx; }
    public int getOutY() { return outy; }

    public String toString() {
        return "(" + ina + "," + inb + ") -> (" + outx + "," + outy + ")";
    }

    public int hashCode() {
        return (ina << 16) + (ina >>> 16) + inb;
    }

    public boolean equals(Object o) {
        if (o == null || !(o instanceof TextPair))
            return false;
        TextPair other = (TextPair) o;
        if (ina != other.ina || inb != other.inb)
            return false;
        int ooutx = other.outx;
        if (outx != ooutx && outx != UNKNOWN && ooutx != UNKNOWN)
            return false;
        int oouty = other.outy;
        if (outy != oouty && outy != UNKNOWN && oouty != UNKNOWN)
            return false;
        return true;
    }
}
