public class TextSource {
    /** Number of chosen plaintexts used. */
    private long nchosen = 0;

    /** Number of known plaintexts used. */
    private long nknown = 0;

    /** Number of elements in the finite field (or entries in each table). */
    private int m;

    /** The cipher object used to make known or chosen plaintexts. */
    kBit cipher;

    /** Creates a TextSource for the given cipher. */
    public TextSource(kBit cipher) {
        this.cipher = cipher;
        m = cipher.getM();
    }

    /** Returns the number of elements in the field. */
    public int getM() { return m; }

    /** Returns the primitive polynomial. */
    public int getP() { return cipher.getP(); }

    /** Returns the number of bits in a cipher block. */
    public int getK() { return cipher.getK(); }

    /** Returns the number of known plaintexts used so far. */
    public long getNKnown() { return nknown; }

    /** Returns the number of chosen plaintexts used so far. */
    public long getNChosen() { return nchosen; }

    /** Make a random known plaintext, and corresponding ciphertext. */
    public TextPair makeKnownPlaintext(TextPair buf) {
        if (buf == null) buf = new TextPair();
        nknown++;
        int ina = RNG.rand0thruN(m-1, m);
        int inb = RNG.rand0thruN(m-1, m);
        cipher.setInput(ina, inb);
        int outx = cipher.getOutX();
        int outy = cipher.getOutY();
        buf.set(ina, inb, outx, outy);
        return buf;
    }

    /** Make a chosen plaintext, and corresponding ciphertext. */
    public TextPair makeChosenPlaintext(int ina, int inb, TextPair buf) {
        if (buf == null) buf = new TextPair();
        nchosen++;
        cipher.setInput(ina, inb);
        int outx = cipher.getOutX();
        int outy = cipher.getOutY();
        buf.set(ina, inb, outx, outy);
        return buf;
    }
}
